@@ -7,7 +7,7 @@ use arkworks_utils::poseidon::{PoseidonError, PoseidonParameters};
 pub mod constraints;
 pub mod field_hasher;
 
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub struct CRH<F: PrimeField>(PhantomData<F>);
 
 impl<F: PrimeField> CRH<F> {
